import yaml
from typing import List

from .filters import FilterConfig

_empty_list = ()


class AnkimakerConfig(yaml.YAMLObject):
    yaml_tag = '!ankimakerconfig'
    header = None
    question_column = None
    answer_column = None
    separators = ','
    input_language = None,
    output_language = None,
    filters: List[List[FilterConfig]] = list()

    def __init__(
            self, separators=',', header=None, answer_column=None, question_column=None,
            filters=tuple(), input_language=None, output_language=None, *args, **karhs
    ):
        self.answer_column = answer_column
        self.question_column = question_column
        self.header = header
        self.separators = separators
        self.input_language = input_language
        self.output_language = output_language
        self.filters = _conditionally_create_new_filters(filters)

    @staticmethod
    def loader(configuration_content):
        if isinstance(configuration_content, dict):
            content = configuration_content['AnkimakerConfig']
        else:
            content = configuration_content
        AnkimakerConfig.header = content.header
        AnkimakerConfig.question_column = content.question_column
        AnkimakerConfig.answer_column = content.answer_column
        AnkimakerConfig.separators = content.separators
        AnkimakerConfig.input_language = content.input_language
        AnkimakerConfig.output_language = content.output_language
        AnkimakerConfig.filters = _conditionally_create_new_filters(content.filters)


def _conditionally_create_new_filters(filters):
    conf_has_filters = len(filters) > 0
    if conf_has_filters:
        should_cast_filter = not isinstance(filters[0][0], FilterConfig)
        if should_cast_filter:
            new_filters = [
                [FilterConfig(**x) for x in or_filter]
                for or_filter in filters
            ]
        else:
            new_filters = filters
        return new_filters
    return list()
