from multiprocessing import Pool
from itertools import repeat
from typing import Iterable, Optional
from http.client import RemoteDisconnected as HttpClientRemoteDisconnected

from PyMultiDictionary import MultiDictionary


def get_and_process_word_definition(language: str, word: str) -> Optional[str]:
    try:
        dictionary = MultiDictionary()
        definition = dictionary.meaning(lang=language, word=word)
        if len(definition[1]) <= 1:
            return None
        definition = definition[1].split('.')[0]
    except HttpClientRemoteDisconnected:
        return None
    return definition


def get_word_definitions_from_dictionary(language: str, word_collection: Iterable[str]) -> Iterable[str]:
    with Pool(7) as p:
        definitions = p.starmap(get_and_process_word_definition, zip(repeat(language), word_collection))
    return definitions
